/* JSCaptions
 * https://bitbucket.org/tagawa/jscaptions
 * 
 * Based on a WebSRT parser written by Philip Jägenstedt (@foolip):
 * http://people.opera.com/philipj/2010/07/21/html5-video-webinar/demos/track.html
 * 
 * TODO:
 *   Make the video playable (without captions) in Mobile Safari.
 *   Make it work with more than one video in a page.
 *   Handle multilingual tracks based on browser language.
 *   Make it more easily configurable.
 */

function parseSrt(srtText, tScale, tOffset, tMargin) {
    tScale = tScale || 1;
    tOffset = tOffset || 0;

    var lines = srtText.split(/\r?\n/);
    var captions = [];
    var timerex = /^(\d\d|\d):(\d\d):(\d\d),(\d{3}) --\> (\d\d|\d):(\d\d):(\d\d),(\d{3})/;
    var lineslen = lines.length;
    var i = 0;
    var t;
    while (i < lineslen) {
        t = timerex.exec(lines[i]);
        if (t) {
            var tStart = 3600 * t[1] + 60 * t[2] + 1 * t[3] + parseFloat('.' + t[4]);
            var tStop = 3600 * t[5] + 60 * t[6] + 1 * t[7] + parseFloat('.' + t[8]);
            if (++i < lineslen) {
                captions.push({'start': tScale * (tStart + tOffset),
                'stop': tScale * (tStop + tOffset) - tMargin, // tMargin allows for transitions (fade in/out, etc.)
                'text': lines[i]});
            }
        }
        i++;
    }
    return captions;
}

function setCapStyles(element) {
    var cap_style = element.style;
    var transition = 'opacity 0.1s linear';
    cap_style.position = 'absolute';
    cap_style.bottom = '30px'; // Allow space for the video controls
    cap_style.color = '#ffffff';
    cap_style.opacity = 0;
    cap_style.textAlign = 'center';
    cap_style.textShadow = '1px 1px 1px #000000';
    cap_style.left = '0.5em';
    cap_style.right = '0.5em';
    cap_style.fontWeight = 'normal';
    cap_style.fontFamily = 'helvetica, arial, sans-serif';
    cap_style.OTransition = transition;
    cap_style.MozTransition = transition;
    cap_style.WebkitTransition = transition;
    cap_style.transition = transition;
}

function initCaptions() {
    var v = document.querySelector('video');
    // Create container div
    var container = document.createElement('div');
    container.style.position = 'relative';
    v.parentNode.insertBefore(container, v);
    container.appendChild(v); // Mobile Safari doesn't like this

    // Create caption div
    var caption = document.createElement('div');
    caption.setAttribute('class', 'jsc_caption');
    setCapStyles(caption);
    container.appendChild(caption);

    // Set the container to the video's width & height
    v.addEventListener('canplay', function() {
        var videoh = v.videoHeight;
        container.style.width = v.videoWidth + 'px';
        container.style.height = videoh + 'px';
        caption.style.fontSize = (videoh / 15) + 'px';
    }, false);

    // Parse subtitles
    var track = document.querySelector('track');
    var srtUrl = track.getAttribute('src');
    var xhr = new XMLHttpRequest();
    xhr.open('GET', srtUrl, false);
    xhr.send();
    var captions = parseSrt(xhr.responseText, 1, 0, 0.1);

    function updateCaption() {
        var now = v.currentTime;
        var currentText = null;
        for (var c, i = 0; c = captions[i]; i++) {
            if (now >= c.start && now <= c.stop) {
                currentText = c.text;
                break;
            }
        }
        if (!currentText) {
            // Hide caption (make it transparent)
            caption.style.opacity = 0;
        } else if (currentText !== caption.textContent) {
            // Show caption (make it opaque)
            caption.textContent = currentText;
            caption.style.opacity = 1;
        }
    }
    function updateLoop() {
        updateCaption();
        if (!v.paused) {
            window.setTimeout(updateLoop, 10);
        }
    }
    v.addEventListener('play', updateLoop, false);
    v.addEventListener('seeked', updateCaption, false);
}

document.addEventListener('DOMContentLoaded', function() {
    if (!document.createElement('track').track) {
        initCaptions();
    }
}, false);
